
/*
 * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
 */

#ifdef __PORTABLE_OBJC__

#ifndef __objcrt__
#define __objcrt__

#include <stdio.h>  /* FILE */
#include <stddef.h> /* size_t */

#ifndef OBJCRTEXPORT
#define OBJCRTEXPORT /* empty */
#endif

#if OBJCRT_PTHREADS
#include <pthread.h>  /* POSIX 1003.1c thread-safe messenger */
#endif
#if OBJCRT_CTHREADS
#include <cthreads.h> /* Mach C thread-safe messenger */
#endif
#if OBJCRT_BOEHM
#include <gc.h>       /* objcInit() vectors to use Hans-J. Boehm gc */
#endif

/* The "id" type is assumed to be defined by the compiler
 * (in the case of objc or objcc, by emitting a typedef)
 * This also allows for cross compiles with gcc in ObjC mode
 * because a definition here would redefine its builtin "id".
 */

/* The traditional Objective C types (see Brad Cox' book)
 */

typedef char *SEL;	/* Uniqued String for Selector */
typedef char *STR;	/* C, NULL-terminated, String */
typedef char BOOL;	/* Boolean */
typedef FILE *IOD;      /* I/O Device */
typedef id SHR;         /* type of class, for us, it's id */

#ifdef __cplusplus
typedef id (*IMP)(...);	/* Method pointer */
#else
typedef id (*IMP)();	/* Method pointer */
#endif

/* The traditional Objective C defines 
 */

#ifndef YES
#define YES (BOOL)1 /* Boolean TRUE */
#endif

#ifndef NO
#define NO (BOOL)0 /* Boolean FALSE */
#endif

#ifndef nil
#define nil (id)0 /* id of Nil instance */
#endif

/* C Messenger interface - try to be compatible */

/* msg tracing */
extern BOOL msgFlag;
extern void OBJCRTEXPORT setMsgFlag(BOOL);
extern FILE *msgIOD;
extern void OBJCRTEXPORT setMsgIOD(FILE*);

extern BOOL allocFlag;   /* allocation tracing */
extern BOOL dbgFlag;     /* verbose dbg() calls */
extern BOOL noCacheFlag; /* disable messenger caching */
extern BOOL noNilRcvr;   /* do not allow msg. to nil */

/* all in the name of backw. compat. */
extern FILE *dbgIOD;
#ifndef NAMEOF
#define NAMEOF(myObject) ([myObject name])
#endif

SEL OBJCRTEXPORT selUid(STR);
STR OBJCRTEXPORT selName(SEL);
void OBJCRTEXPORT dbg(char *fmt, ...);
void OBJCRTEXPORT addObjcModules(void *modPtr);

IMP OBJCRTEXPORT _imp(id,SEL);
IMP OBJCRTEXPORT _impSuper(id,SEL);

#define OC_STOREON_VEC		_storeOn
#define OC_READFROM_VEC		_readFrom

extern id (*_fileIn)(FILE*);
extern BOOL (*_fileOut)(FILE*,id);
void OBJCRTEXPORT setFileIn(id (*f)(FILE *));
void OBJCRTEXPORT setFileOut(BOOL (*f)(FILE *,id));
extern BOOL (*OC_STOREON_VEC)(STR,id);  
extern id (*OC_READFROM_VEC)(STR);

extern id (*_showOn)(id,unsigned); 

extern void *(*objcrt_malloc)(size_t);
extern void *(*objcrt_malloc_atomic)(size_t);
extern void *(*objcrt_realloc)(void*,size_t);
extern void (*objcrt_free)(void*);

void OBJCRTEXPORT *OC_Malloc(size_t);
/* allocate memory from the system */
void OBJCRTEXPORT *OC_MallocAtomic(size_t); 
/* same as OC_Malloc, but contains no pointers */
void OBJCRTEXPORT *OC_Calloc(size_t); 
/* same as OC_Malloc, but all zeroes */
void OBJCRTEXPORT *OC_Realloc(void *,size_t); 
/* reallocate memory from the system */
void OBJCRTEXPORT *OC_Free(void *data); 
/* deallocate OC_Malloc'ed memory and return NULL */

#if OBJCRT_PTHREADS || OBJCRT_CTHREADS
extern void (*objcrt_cache_lock)(void);
extern void (*objcrt_cache_unlock)(void);
#endif

#endif /* __objcrt__ */

#endif /* __PORTABLE_OBJC__ */


