
/*
 * Portable Object Compiler (c) 1997, 98.  All Rights Reserved.
 */

#ifdef __PORTABLE_OBJC__

#ifndef __OBJECT_INCLUDED__
#define __OBJECT_INCLUDED__
#include "Object.h" /* Stepstone Object.h assumes #import */
#endif

#include "Block.h" /* ifError: stuff */

@implementation Object
/*****************************************************************************
 *
 * Factory Initialization
 *
 ****************************************************************************/

+ initialize
{ 
	return self; 
}

- initialize
{
	return self;
}

/*****************************************************************************
 *
 * Creating, Copying and Freeing
 *
 ****************************************************************************/


- str:(STR)s
{
    return [self shouldNotImplement]; /* dummy def. Stepstn */
}

+ new
{
	id newObject = (*OC_ALLOC_VEC)(self, 0);return newObject;
}

- new
{
	return [self notImplemented:_cmd];
}

- copy
{
	id newObject = (*OC_COPY_VEC)(self, 0);return newObject;
}

- deepCopy
{
	id newObject = (*OC_COPY_VEC)(self, 0);return newObject;
}

- free
{
        isa = nil; return (OC_DEALLOC_VEC)?(*OC_DEALLOC_VEC)(self):nil; 
}

+ free
{ 
	return nil; 
}

/*****************************************************************************
 *
 * Identity
 *
 ****************************************************************************/


- self
{
    return self;
}

- yourself
{
    return self;
}

- class
{
    return [isa class];
}

- superclass
{
    return [isa superclass];
}

- superClass
{
    return [isa superClass];
}

+ class
{
    return self;
}

+ superclass
{
       return (id) ((struct objcrt_shared *)self)->clsSuper; 
}

+ superClass
{
       return (id) ((struct objcrt_shared *)self)->clsSuper; 
}

- add:anObject
{
    return [self notImplemented:_cmd]; /* mmh */
} 

- (STR) name
{
	return [isa name];
}

+ (STR) name
{
	return ((struct objcrt_shared *)self)->clsName; 
}


- findClass:(STR)aClassName
{ 
	return (*OC_CVT_TO_ID_VEC)(aClassName);
}

- (SEL) selOfSTR:(STR)aSelName
{

	SEL aSel = (*OC_CVT_TO_SEL_VEC)(aSelName);

	if (aSel) {
	    return aSel;
	} else {
	    [self error:"Selector not in hashtable."];
	    return NULL;
	}
}

- idOfSTR:(STR)aClassName
{
	id aClass = [self findClass:aClassName];
	if (aClass) {
	    return aClass;
	} else {
	    return [self error:"Class not linked in application."];
	}
}

/*****************************************************************************
 *
 * Comparing
 *
 ****************************************************************************/


- (unsigned) hash
{
	/* MIPSpro 64bit : cast a size_t to unsigned and avoid a warning
	* it is not incorrect for |hash| to ignore the upper 32bit
	*/

	return (unsigned) (self - nil);
}

- (BOOL) isEqual:anObject
{
	return (BOOL) (anObject == self); 
}

+ (BOOL) isEqual:anObject
{
	return (BOOL) (self == anObject);
}

- (BOOL) isSame:anObject
{ 
	return (BOOL) (self == anObject);
}

- (BOOL) notEqual:anObject
{
	return [self isEqual:anObject] == NO;
}

- (BOOL) notSame:anObject
{ 
	return (BOOL) ![self isSame:anObject]; 
}

- (int) compare:anObject
{
	[self notImplemented:_cmd];return 0;
}

/*****************************************************************************
 *
 * Class and Methods
 *
 ****************************************************************************/


- (BOOL) respondsTo:(SEL)aSelector
{
	unsigned n; 
	struct objcrt_slt *smt; 
	struct objcrt_shared * cls;
	
	cls = (struct objcrt_shared *) isa; 

	do {
		for (smt = cls->clsDispTable, n = cls->clsSizDict; n--; smt++) {
			/* selectors can be compared with == */
			if (smt->_cmd == aSelector) return YES;
		}
	} while ((cls = cls->clsSuper));

	return NO;
}

- (BOOL) isMemberOf:aClass
{
	return (BOOL) (isa == aClass);
}

- (BOOL) isKindOf:aClass
{
	struct objcrt_shared *cls = (struct objcrt_shared *) isa;
	struct objcrt_shared *aCls = (struct objcrt_shared *) aClass;

	for(;cls;cls = cls->clsSuper) if (cls == aCls) return YES;

	return NO;
}

/*****************************************************************************
 *
 * Class Management
 *
 ****************************************************************************/

+ subclasses
{
    id c = [[Object idOfSTR:"OrdCltn"] new];
    addSubclassesTo(c,[self name]);
    return c;
}
+ poseAs:superClass
{
    objcrt_poseAs((struct objcrt_shared *)self,(struct objcrt_shared *)superClass);
    return self;
}

+ addMethodsTo:superClass
{
    objcrt_addMethodsTo((struct objcrt_shared *)self,(struct objcrt_shared *)superClass);
    return self;
}

+ subclass:(STR)name
{
    return (id) objcrt_subclass(name,(struct objcrt_shared *)self,0,0);
}

+ (BOOL) inheritsFrom:aClass
{
    BOOL x;
    x=objcrt_inheritsFrom((struct objcrt_shared *)self,(STR)[aClass name]);
    return x;
}

/*****************************************************************************
 *
 * Error Handling
 *
 ****************************************************************************/


- subclassResponsibility:(SEL)aSelector
{
    return [self error:"Subclass should have implemented one of my methods."];
}

- subclassResponsibility
{
    return [self subclassResponsibility:"(Unknown method)"];
}

- notImplemented:(SEL)aSelector
{
    return [self error:"Does not implement this message."];
}

- notImplemented
{
	return [self notImplemented:"(Unknown method)"];
}

- shouldNotImplement:(SEL)aSelector
{
    return [self shouldNotImplement:aSelector from:isa];
}

- shouldNotImplement
{
    return [self shouldNotImplement:"(Unknown method)"];
}

- shouldNotImplement:(SEL)aSelector from:superClass
{
    [self error:"Message is not approriate for this class."];
    return self;
}

- doesNotRecognize:(SEL)aSelector
{
    [self error:"Message not recognized by this class."];
    return self;
}
- doesNotUnderstand:(SEL)aSelector
{
    return [self doesNotRecognize: aSelector];
}

- vsprintf:(STR)format:(OC_VA_LIST)ap
{
    return [self shouldNotImplement];
}

- error:(STR)format,...
{
	OC_VA_LIST ap;
        static id MsgClass;
	if (!MsgClass) MsgClass=[self findClass:"String"];

	/* use OC macros here for porting to SunOS4 */

	OC_VA_START(ap,format);
	if (MsgClass) {
	    [self halt:[MsgClass vsprintf:format:ap]];
	} else {
	    (*OC_ERROR_VEC)(self,format, ap); 
	}
	OC_VA_END(ap);

	return self;
}

- halt:message
{
    id handler = [Block pop]; 

    if (handler) {
	[handler value:message value:self];
	[Block push:handler];
    } else {
        handler = [Block errorHandler];
	[handler value:message value:self];
    }

    return self;
}

/*****************************************************************************
 *
 * Method Implemenation Lookup
 *
 ****************************************************************************/


- (IMP) methodFor:(SEL)aSelector
{
	return _imp(self,aSelector);
}

+ (IMP) instanceMethodFor:(SEL)aSelector
{
	return _impSuper(self,aSelector);
}

/*****************************************************************************
 *
 * Method Performing
 *
 ****************************************************************************/


- perform:(SEL)aSelector
{
   /* the usual all C cast */
   return (*(id(*)(id,SEL))_imp(self, aSelector))(self, aSelector);
}

- perform:(SEL)aSelector with:anObject
{
   /* the usual all C cast */
   return (*(id(*)(id,SEL,id))_imp(self, aSelector))(self,aSelector,anObject);
}

- perform:(SEL)aSelector with:anObject with:otherObject
{
   /* the usual all C cast */
   return (*(id(*)(id,SEL,id,id))_imp(self, aSelector))(self,aSelector,anObject,otherObject);
}

- perform:(SEL)aSelector with:anObject with:otherObject with:thirdObj
{
   /* the usual all C cast */
   return (*(id(*)(id,SEL,id,id,id))_imp(self, aSelector))(self,aSelector,anObject,otherObject,thirdObj);
}
/*****************************************************************************
 *
 * Printing
 *
 ****************************************************************************/

- print
{
    return [self printOn:stdout];
}

+ print
{
    printf([self name]);
    return self;
}


- printLine
{
    [self print];
    printf("\n");
    return self;
}

- show
{
	return (*_showOn)(self, 0); 
}

- printToFile:(FILE *)aFile
{
    /* THIS METHOD WILL GO AWAY ! */

    /* it used to be precisely the opposite -- printOn was defined
     * in terms of our printToFile, but we decided against this
     * invention of ourselves (other runtimes don't have printToFile)
     * because subclasses sometimes already override printOn:
     * (cfr. Stepstone runtime)
     */

     return [self printOn:aFile];
}

- printOn:(IOD)anIOD
{
    return self;
}

/*****************************************************************************
 *
 * Version
 *
 ****************************************************************************/

+ (STR) objcrtRevision
{
    return __objcrt_revision__;
}

/*****************************************************************************
 *
 * AsciiFiler Methods for Object Class
 *
 ****************************************************************************/

+ readFrom:(STR)aFileName
{ 
	return (*_readFrom)(aFileName); 
}
- (BOOL) storeOn:(STR)aFileName
{ 
	return (*_storeOn)(aFileName, self); 
}
/*****************************************************************************
 *
 * AsciiFiler Methods for Object Class
 *
 ****************************************************************************/

- fileOutOn:aFiler
{
	[aFiler fileOut:& isa type:'#'];
	return [self fileOutIdsFor:aFiler];
}
+ fileInFrom:aFiler
{
	id newObject = (*OC_ALLOC_VEC)(self, 0);
	return [newObject fileInFrom:aFiler];
}

- fileInFrom:aFiler
{
	return [self fileInIdsFrom:aFiler];
}
- awake
{
	return self; 
}

- awakeFrom:aFiler
{
	return [ self awake ];
}


@end

#endif /* __PORTABLE_OBJC__ */


