
/*
 * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
 */

#include "objpak.h"

@implementation Cltn

- eachElement
{
    return [self subclassResponsibility:_cmd];
}

- (BOOL) includes:anObject
{
    [self subclassResponsibility:_cmd];
    return NO;
}


- add:anObject
{
    return [self subclassResponsibility:_cmd];
}

- remove:anObject
{
    return [self subclassResponsibility:_cmd];
}

- addYourself
{
    return [self subclassResponsibility:_cmd];
}

- emptyYourself
{
    return [self subclassResponsibility:_cmd];
}

- perform:(SEL)aSel with:a with:b with:c
{
    /* NeXT Object.h doesn't have this, so add this dummy */
    return [self subclassResponsibility:_cmd];
}

/*****************************************************************************
 *
 * Creating with contents.
 *
 ****************************************************************************/

+ with:(int)nArgs,...
{
    id newObject;

    /* use OC macros for porting to SunOS4 */
    OC_VA_LIST vp;

    newObject = [self new];

    /* #if 0 this piece of code if problems with stdarg
     * typically this means the driver is not configured with
     * the right -builtintype
     * or builtinfunction flags (because the macros might expand to these)
     * 
     * alternative solution: check the .P output (-retain) and 
     * do a setenv OBJCOPT -builtinfunction __builtin_foo
     * (and please let me know)
     */

#if 1
    OC_VA_START(vp,nArgs);
    while (nArgs-- > 0) {
		id anObject = OC_VA_ARG(vp,id);
		[newObject add:anObject];
	}
    OC_VA_END(vp);
#endif

    return newObject;
}

+ with:firstObject with:nextObject
{
    return [[[self new] add:firstObject] add:nextObject];
}

+ add:firstObject
{
    return [[self new] add:firstObject];
}

/*****************************************************************************
 *
 * Testing Contents
 *
 ****************************************************************************/

- (BOOL) includesAllOf:aCol
{
    if (self == aCol) {
	return YES;
    } else {
	id e,seq = [aCol eachElement];
	while ((e=[seq next])) if (![self includes:e]) {
	    [seq free];return NO;
	}
	[seq free];return YES;
    }
}

- (BOOL) includesAnyOf:aCol
{
    if (self == aCol) {
	return YES;
    } else {
	id e,seq = [aCol eachElement];
	while ((e=[seq next])) if ([self includes:e]) {
	    [seq free];return YES;
	}
	[seq free];return NO;
    }
}

/*****************************************************************************
 *
 * Adding and Removing Contents
 *
 ****************************************************************************/

- addContentsTo:aCol
{
    if (self == aCol) {
	[self addYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [self eachElement];
	while ((anElement = [elements next])) [aCol add:anElement];
	elements = [elements free];
    }
	
    return aCol;
}

- addContentsOf:aCol
{
    if (self == aCol) {
	[self addYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [aCol eachElement];
	while ((anElement = [elements next])) [self add:anElement];
	elements = [elements free];
    }
	
    return self;
}
- addAll:aCol
{
    return [self addContentsOf:aCol];
}

- removeContentsOf:aCol
{
    if (self == aCol) {
	return [self emptyYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [aCol eachElement];
	while ((anElement = [elements next])) [self remove:anElement];
	elements = [elements free];
	
	return self;
    }
}

- removeContentsFrom:aCol
{
    if (self == aCol) {
	[self emptyYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [self eachElement];
	while ((anElement = [elements next])) [aCol remove:anElement];
	elements = [elements free];
    }
	
    return self;
}

- removeAll:aCol
{
    return [self removeContentsOf:aCol];
}

/*****************************************************************************
 *
 * Converting to different type of collection.
 *
 ****************************************************************************/

- asSet
{
    /* Stepstone isKindOf takes a id but class returns SHR */
    if ([self isKindOf:(id)[Set class]]) {
	return self;
    } else {
	return [[Set new] addContentsOf:self];
    }
}

- asOrdCltn
{
    /* Stepstone isKindOf takes a id but class returns SHR */
    if ([self isKindOf:(id)[OrdCltn class]]) {
	return self;
    } else {
	return [[OrdCltn new] addContentsOf:self];
    }
}

/*****************************************************************************
 *
 * Block methods
 *
 ****************************************************************************/

- detect:aBlock
{
    id elements;
    id anElement;

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([aBlock value:anElement])) {
	    elements = [elements free];
	    return anElement; 
	}
    }

    elements = [elements free];
    return nil;
}

- detect:aBlock ifNone:noneBlock
{
    id elements;
    id anElement;

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([aBlock value:anElement])) {
	    elements = [elements free];return anElement; 
	}
    }

    elements = [elements free];
    return [noneBlock value];
}

- select:testBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([testBlock value:anElement])) [newObject add:anElement];
    }

    elements = [elements free];
    return newObject;
}

- reject:testBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (!([testBlock value:anElement])) [newObject add:anElement];
    }

    elements = [elements free];
    return newObject;
}

- collect:transformBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	id anImage = [ transformBlock value:anElement ];
	if (anImage) [ newObject add:anImage ]; 
    }

    elements = [elements free];
    return newObject;
}

- (unsigned) count:aBlock
{
    id elements;
    id anElement;
    unsigned c = 0;

    elements = [self eachElement];
    while ((anElement = [elements next])) {
	if ([aBlock value:anElement]) c++;
    }
    elements = [elements free];

    return c;
}

/*****************************************************************************
 *
 * Making elements perform actions.
 *
 ****************************************************************************/

- elementsPerform:(SEL)aSelector
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector with:anObject];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject with:otherObject
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector with:anObject with:otherObject];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject with:otherObject with:thirdObj
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
     [anElement perform:aSelector with:anObject with:otherObject with:thirdObj];
    }
    elements = [elements free];
    
    return self;
}
/*****************************************************************************
 *
 * Do Blocks
 *
 ****************************************************************************/

- do:aBlock
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) [aBlock value:anElement];
    elements = [elements free];
    
    return self;
}

- do:aBlock until:(BOOL*)flag
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
	[aBlock value:anElement];
	if (*flag) break;
    }
    elements = [elements free];
    
    return self;
}

@end

