
/*
 * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
 */

#include "objpak.h"

#ifndef NDEBUG
#define CHECK_BALANCE
#endif

@implementation SortCltn

/*****************************************************************************
 *
 * Creation
 *
 ****************************************************************************/

static objbbt_t objbbt_alloc()
{
    return (objbbt_t)OC_Malloc(sizeof(struct objbbt));
}

static objbbt_t objbbt_init(objbbt_t self,id key)
{
    self->ulink   = NULL;
    self->llink   = NULL;
    self->rlink   = NULL;
    self->key     = key;
    self->balance = 0;
    return self;
}

static objbbt_t objbbt_new(id key)
{
    return objbbt_init(objbbt_alloc(),key);
}

static int objbbt_sign(objbbt_t ulink,objbbt_t self)
{
    assert(ulink->llink == self || ulink->rlink == self);
    return (ulink->llink == self)?-1:+1;
}

static objbbt_t objbbt_slink(objbbt_t self,int sign)
{
    assert(sign == +1 || sign == -1);return (sign>0)?self->rlink:self->llink;
}

static void objbbt_setllink(objbbt_t self,objbbt_t node)
{
    self->llink = node;if (node) node->ulink = self;
}

static void objbbt_setrlink(objbbt_t self,objbbt_t node)
{
    self->rlink = node;if (node) node->ulink = self;
}

static void objbbt_setslink(objbbt_t self,objbbt_t node,int sign)
{
    if (sign>0) objbbt_setrlink(self,node); else objbbt_setllink(self,node);
}

static void objbbt_freeobjects(objbbt_t self)
{
    if (self->llink) objbbt_freeobjects(self->llink);
    if (self->rlink) objbbt_freeobjects(self->rlink);
    self->key = [self->key free];
}

static objbbt_t objbbt_free(objbbt_t self)
{
    if (self->llink) self->llink = objbbt_free(self->llink);
    if (self->rlink) self->rlink = objbbt_free(self->rlink);
    OC_Free(self);return NULL;
}

+ new
{
    return [self newCmpSel:@selector(compare:)];
}

+ new:(unsigned)n
{
    return [self new];
}

+ newDictCompare
{
    return [self newCmpSel:@selector(dictCompare:)];
}

- setUpCmpBlock:sortBlock
{
    cmpBlk = sortBlock;
    objbbt_init(&value,(id)0xdeadbeaf);
    return self;
}

+ sortBlock:sortBlock
{
    id newObj = [super new];[newObj setUpCmpBlock:sortBlock];return newObj;
}

+ sortBy:sortBlock
{
    return [self sortBlock:sortBlock];
}

- setUpCmpSel:(SEL)aSel
{
    cmpSel = aSel;
    objbbt_init(&value,(id)0xdeadbeaf);
    return self;
}

+ newCmpSel:(SEL)aSel
{
    id newObj = [super new];[newObj setUpCmpSel:aSel];return newObj;
}

/*****************************************************************************
 *
 * Creating with contents.
 *
 ****************************************************************************/

+ with:(int)nArgs,...
{
    id newObject;

    /* use OC macros for porting to SunOS4 */
    OC_VA_LIST vp;

    newObject = [self new];

    /* #if 0 this piece of code if problems with stdarg
     * typically this means the driver is not configured with
     * the right -builtintype
     * or builtinfunction flags (because the macros might expand to these)
     * 
     * alternative solution: check the .P output (-retain) and 
     * do a setenv OBJCOPT -builtinfunction __builtin_foo
     * (and please let me know)
     */

#if 1
    OC_VA_START(vp,nArgs);
    while (nArgs-- > 0) {
		id anObject = OC_VA_ARG(vp,id);
		[newObject add:anObject];
	}
    OC_VA_END(vp);
#endif

    return newObject;
}

+ with:firstObject with:nextObject
{
    return [[[self new] add:firstObject] add:nextObject];
}

+ add:firstObject
{
    return [[self new] add:firstObject];
}

- copy
{
    return [[isa new] addContentsOf:self];
}

- deepCopy
{
    id aSeq,elt;
    id aCopy = [isa new];
    
    aSeq = [self eachElement];
    while ((elt = [aSeq next])) [aCopy add:[elt deepCopy]];
    aSeq = [aSeq free];

    return aCopy;
}

- emptyYourself
{
    if (value.llink) value.llink = objbbt_free(value.llink);
    return self;
}

- freeContents
{
    if (value.llink) {
	objbbt_freeobjects(value.llink);
	value.llink = objbbt_free(value.llink);
    }
    return self;
}

- free
{
    if (value.llink) value.llink = objbbt_free(value.llink);
    return [super free];
}

/*****************************************************************************
 *
 * Interrogation
 *
 ****************************************************************************/

- (objbbt_t) objbbtTop
{
    return value.llink;
}

- (SEL) comparisonSelector
{
    return cmpSel;
}

static int objbbt_size(objbbt_t self)
{
    int size = 1;
    if (self->llink) size += objbbt_size(self->llink);
    if (self->rlink) size += objbbt_size(self->rlink);
    return size;
}

- (unsigned) size
{
    return (value.llink)?objbbt_size(value.llink):0;
}

- (BOOL) isEmpty
{
    return value.llink == NULL;
}

- eachElement
{
    id aCarrier = [TreeSequence over:self];
	return [Sequence over:aCarrier];
}

/*****************************************************************************
 *
 * Comparing
 *
 ****************************************************************************/

- (unsigned) hash
{
    [self notImplemented:_cmd]; return 0;
}

- (BOOL) isEqual:aSort
{
    [self notImplemented:_cmd]; return NO;
}

/*****************************************************************************
 *
 * Adding
 *
 ****************************************************************************/

static int objbbt_cmp(objbbt_t self,id key,SEL cmpSel,id cmpBlk,objbbt_t *offset)
{
    int cmp;
    objbbt_t link;

    /* suppress warning for cast to int on 64bit MIPSpro */

    if (cmpBlk) {
	cmp = (int) ( [ cmpBlk value:key value:self->key ] - nil );
    } else {
        cmp = (int) ( [ key perform:cmpSel with:self->key ] - nil );
    }
    
    if (cmp == 0) { *offset = self;return cmp; }
    link = (cmp < 0)?self->llink:self->rlink;
    
    return (link)?objbbt_cmp(link,key,cmpSel,cmpBlk,offset):(*offset=self,cmp);
}

static int objbbt_cmpne(objbbt_t self,id key,SEL cmpSel,id cmpBlk,objbbt_t *offset)
{
    int cmp;
    objbbt_t link;

    /* suppress warning for cast to int on 64bit MIPSpro */

    if (cmpBlk) {
	cmp = (int) ( [ cmpBlk value:key value:self->key ] - nil );
    } else {
        cmp = (int) ( [ key perform:cmpSel with:self->key ] - nil );
    }

    if (cmp == 0) { cmp = +1; }
    link = (cmp < 0)?self->llink:self->rlink;

    return (link)?objbbt_cmpne(link,key,cmpSel,cmpBlk,offset):(*offset=self,cmp);
}

static int objbbt_height(objbbt_t self)
{
    if (self) {
        int a,b;
	a = objbbt_height(self->llink);
	b = objbbt_height(self->rlink);
	assert(self->balance == (b - a));
	return 1 + ((a>b)?a:b);
    } else {
	return 0;
    }
}

static objbbt_t objbbt_bnode(objbbt_t top,objbbt_t newp)
{
    while (newp != top && newp->balance == 0) newp = newp->ulink;
    assert(newp->balance != 0 || newp == top);
    return newp;
}

static int objbbt_adjust(objbbt_t bnode,objbbt_t newp)
{
    int sign = 0;
    objbbt_t ulink;
    
    while (1) {
	ulink = newp->ulink;
	sign  = objbbt_sign(ulink,newp);
	if (ulink == bnode) break;
	assert(ulink->balance == 0);ulink->balance = sign;newp = ulink;
    }
    
    return sign;
}

/*
 *                A                       B
 *               / \                     / \
 *              a   B       --->        A   \
 *             /   / \                 / \   c
 *                b   c               a   b   \
 *               /     \             /     \   \
 *                      \
 */

static void objbbt_sglrot(objbbt_t A,objbbt_t B,int sign)
{
    objbbt_t U = A->ulink;
    objbbt_setslink(A,objbbt_slink(B,-sign),sign);
    objbbt_setslink(B,A,-sign);
    A->balance = 0;B->balance = 0;
    objbbt_setslink(U,B,objbbt_sign(U,A));
}
    
/*                                            
 *                A                            X      
 *               / \                        /     \
 *              a   B       --->           A       B   
 *             /   / \                    / \     / \ 
 *                X   d                  a   b   c   d
 *               / \   \                /     \ /     \  
 *              b   c                                
 *             /     \
 */

static void objbbt_dblrot(objbbt_t A,objbbt_t B,int sign)
{
    objbbt_t U = A->ulink;
    objbbt_t X = objbbt_slink(B,-sign);
    
    objbbt_setslink(B,objbbt_slink(X,sign),-sign);
    objbbt_setslink(X,B,+sign);
    objbbt_setslink(A,objbbt_slink(X,-sign),sign);
    objbbt_setslink(X,A,-sign);
    
    if (X->balance == +sign) { A->balance = -sign; B->balance =     0; }
    if (X->balance ==     0) { A->balance =     0; B->balance =     0; }
    if (X->balance == -sign) { A->balance =     0; B->balance = +sign; }
    
    X->balance = 0;objbbt_setslink(U,X,objbbt_sign(U,A));
}

static void objbbt_rot(objbbt_t A,int sign)
{
    objbbt_t B = objbbt_slink(A,sign);
    assert(sign == A->balance && sign != 0 && B->balance != 0);
    /* SGI cc doesn't like return of a 'void' function */
    if (sign == +B->balance) { objbbt_sglrot(A,B,sign);return; }
    if (sign == -B->balance) { objbbt_dblrot(A,B,sign);return; }
}

static void objbbt_rebalance(objbbt_t top,objbbt_t newp)
{
    int sign;
    objbbt_t bnode;
    
    bnode = objbbt_bnode(top,newp);
    sign  = objbbt_adjust(bnode,newp);
    
    if (bnode->balance == 0)     { bnode->balance = sign;return;  }
    if (bnode->balance == -sign) { bnode->balance = 0;return;     }
    if (bnode->balance == +sign) { objbbt_rot(bnode,sign);return; }
}

static void objbbt_addfirst(objbbt_t self,id key)
{
    objbbt_setllink(self,objbbt_new(key));
}

static void objbbt_addat(objbbt_t top,id key,int cmp,objbbt_t offset)
{
    objbbt_t newp = objbbt_new(key);
    
    assert(cmp < 0 || cmp > 0);
    if (cmp < 0) objbbt_setllink(offset,newp);
    if (cmp > 0) objbbt_setrlink(offset,newp);
    
    objbbt_rebalance(top,newp);
    
#ifdef CHECK_BALANCE
    assert(objbbt_height(top) > 0);
#endif
}

static void objbbt_add(objbbt_t top,id key,SEL selCmp,id cmpBlk)
{
    objbbt_t offset = NULL;
    int cmp = objbbt_cmpne(top,key,selCmp,cmpBlk,&offset);
    objbbt_addat(top,key,cmp,offset);
}

- add:anObject
{
    if (anObject) {
	if (value.llink) {
	    objbbt_add(value.llink,anObject,cmpSel,cmpBlk);
	    return self;
	} else {
	    objbbt_addfirst(&value,anObject);
	    return self;
	}
    } else {
	return nil;
    }
}

static id objbbt_addnfind(objbbt_t top,id key,SEL selCmp,id cmpBlk)
{
    objbbt_t offset = NULL;
    int cmp = objbbt_cmpne(top,key,selCmp,cmpBlk,&offset);
    if (cmp == 0) {
	return offset->key;
    } else {
	objbbt_addat(top,key,cmp,offset);
	return key;
    }
}

- addNTest:anObject
{
    if (anObject) {
	if (value.llink) {
	    id res = objbbt_addnfind(value.llink,anObject,cmpSel,cmpBlk);
	    return (res == anObject)?anObject:nil;
	} else {
	    objbbt_addfirst(&value,anObject);
	    return anObject;
	}
    } else {
	return nil;
    }
}

- filter:anObject
{
    if (anObject) {
	if (value.llink) {
	    id res = objbbt_addnfind(value.llink,anObject,cmpSel,cmpBlk);
	    return (res == anObject)?anObject:([anObject free],res);
	} else {
	    objbbt_addfirst(&value,anObject);
	    return anObject;
	}
    } else {
	return nil;
    }
}

static id objbbt_replace(objbbt_t top,id key,SEL selCmp,id cmpBlk)
{
    objbbt_t offset = NULL;
    int cmp = objbbt_cmpne(top,key,selCmp,cmpBlk,&offset);
    if (cmp == 0) {
	id tmp = offset->key;offset->key = key;return tmp;
    } else {
	objbbt_addat(top,key,cmp,offset);
	return nil;
    }
}

- replace:anObject
{
    if (anObject) {
	if (value.llink) {
	    return objbbt_replace(value.llink,anObject,cmpSel,cmpBlk);
	} else {
	    objbbt_addfirst(&value,anObject);
	    return nil;
	}
    } else {
	return nil;
    }
}

/*****************************************************************************
 *
 * Removing
 *
 ****************************************************************************/

- remove:oldObject
{
    if (oldObject) {
	return [self notImplemented:_cmd]; 
    } else {
	return nil;
    }
}

/*****************************************************************************
 *
 * Testing Contents
 *
 ****************************************************************************/

- (BOOL) includesAllOf:aCol
{
    if (self == aCol) {
	return YES;
    } else {
	id e,seq = [aCol eachElement];
	while ((e=[seq next])) if (![self includes:e]) {
	    [seq free];return NO;
	}
	[seq free];return YES;
    }
}

- (BOOL) includesAnyOf:aCol
{
    if (self == aCol) {
	return YES;
    } else {
	id e,seq = [aCol eachElement];
	while ((e=[seq next])) if ([self includes:e]) {
	    [seq free];return YES;
	}
	[seq free];return NO;
    }
}

/*****************************************************************************
 *
 * Adding and Removing Contents
 *
 ****************************************************************************/

- addContentsTo:aCol
{
    if (self == aCol) {
	[self addYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [self eachElement];
	while ((anElement = [elements next])) [aCol add:anElement];
	elements = [elements free];
    }
	
    return aCol;
}

- addContentsOf:aCol
{
    if (self == aCol) {
	[self addYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [aCol eachElement];
	while ((anElement = [elements next])) [self add:anElement];
	elements = [elements free];
    }
	
    return self;
}
- addAll:aCol
{
    return [self addContentsOf:aCol];
}

- removeContentsOf:aCol
{
    if (self == aCol) {
	return [self emptyYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [aCol eachElement];
	while ((anElement = [elements next])) [self remove:anElement];
	elements = [elements free];
	
	return self;
    }
}

- removeContentsFrom:aCol
{
    if (self == aCol) {
	[self emptyYourself];
    } else {
	id elements;
	id anElement;
	
	elements = [self eachElement];
	while ((anElement = [elements next])) [aCol remove:anElement];
	elements = [elements free];
    }
	
    return self;
}

- removeAll:aCol
{
    return [self removeContentsOf:aCol];
}

/*****************************************************************************
 *
 * Converting to different type of collection.
 *
 ****************************************************************************/

- asSet
{
    /* Stepstone isKindOf takes a id but class returns SHR */
    if ([self isKindOf:(id)[Set class]]) {
	return self;
    } else {
	return [[Set new] addContentsOf:self];
    }
}

- asOrdCltn
{
    /* Stepstone isKindOf takes a id but class returns SHR */
    if ([self isKindOf:(id)[OrdCltn class]]) {
	return self;
    } else {
	return [[OrdCltn new] addContentsOf:self];
    }
}

/*****************************************************************************
 *
 * Block methods
 *
 ****************************************************************************/

- detect:aBlock
{
    id elements;
    id anElement;

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([aBlock value:anElement])) {
	    elements = [elements free];
	    return anElement; 
	}
    }

    elements = [elements free];
    return nil;
}

- detect:aBlock ifNone:noneBlock
{
    id elements;
    id anElement;

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([aBlock value:anElement])) {
	    elements = [elements free];return anElement; 
	}
    }

    elements = [elements free];
    return [noneBlock value];
}

- select:testBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (([testBlock value:anElement])) [newObject add:anElement];
    }

    elements = [elements free];
    return newObject;
}

- reject:testBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	if (!([testBlock value:anElement])) [newObject add:anElement];
    }

    elements = [elements free];
    return newObject;
}

- collect:transformBlock
{
    id elements;
    id anElement;
    id newObject = [isa new];

    elements = [self eachElement];

    while ((anElement = [elements next])) {
	id anImage = [ transformBlock value:anElement ];
	if (anImage) [ newObject add:anImage ]; 
    }

    elements = [elements free];
    return newObject;
}

- (unsigned) count:aBlock
{
    id elements;
    id anElement;
    unsigned c = 0;

    elements = [self eachElement];
    while ((anElement = [elements next])) {
	if ([aBlock value:anElement]) c++;
    }
    elements = [elements free];

    return c;
}

/*****************************************************************************
 *
 * Making elements perform actions.
 *
 ****************************************************************************/

- elementsPerform:(SEL)aSelector
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector with:anObject];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject with:otherObject
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
		[anElement perform:aSelector with:anObject with:otherObject];
	}
    elements = [elements free];
    
    return self;
}

- elementsPerform:(SEL)aSelector with:anObject with:otherObject with:thirdObj
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
     [anElement perform:aSelector with:anObject with:otherObject with:thirdObj];
    }
    elements = [elements free];
    
    return self;
}
/*****************************************************************************
 *
 * Do Blocks
 *
 ****************************************************************************/

- do:aBlock
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) [aBlock value:anElement];
    elements = [elements free];
    
    return self;
}
- do:aBlock until:(BOOL*)flag
{
    id elements;
    id anElement;
    
    elements = [self eachElement];
    while ((anElement = [elements next])) {
	[aBlock value:anElement];
	if (*flag) break;
    }
    elements = [elements free];
    
    return self;
}

/*****************************************************************************
 *
 * Locating
 *
 ****************************************************************************/

static id objbbt_find(objbbt_t self,id key,SEL cmpSel,id cmpBlk)
{
    int cmp;
    objbbt_t offset = NULL;
    
    if ((cmp = objbbt_cmp(self,key,cmpSel,cmpBlk,&offset))) {
	return nil;
    } else {
	assert([key isEqual:offset->key]);return offset->key;
    }
}

- find:anObject
{
    if (anObject) {
	return (value.llink)?objbbt_find(value.llink,anObject,cmpSel,cmpBlk):nil;
    } else {
	return nil;
    }
}

- (BOOL) contains:anObject
{
    return (BOOL)([self find:anObject] ? YES : NO); 
}

/*****************************************************************************
 *
 * Printing
 *
 ****************************************************************************/

- printOn:(IOD)aFile
{
    [[[self eachElement] printOn:aFile] free];return self;
}

/*****************************************************************************
 *
 * Archiving
 *
 ****************************************************************************/

- fileOutOn:aFiler
{
    return [self notImplemented:_cmd];
}

- fileInFrom:aFiler
{
    return [self notImplemented:_cmd];
}

@end

