
/*
 * Portable Object Compiler (c) 1997.  All Rights Reserved.
 */

#include "oclib.h"

@implementation Identifier

/*****************************************************************************
 *
 * Creation
 *
 ****************************************************************************/


- setUpString:aString
{
	lineTag = [LineTag currentTag];
	string  = aString;
	return self;
}

- lineTag
{
    return lineTag;
}

+ str:(STR)s
{
     return [self string:[String str:s]];
}

+ string:aString
{
    return [[super new] setUpString:aString];
}

- dontFree
{
    return self;
}

- free
{
    return nil;
}


/*****************************************************************************
 *
 * Equality (necessary for being member of Set)
 *
 ****************************************************************************/


- (int) asInt
{
    return [string asInt]; /* returns 0 for symbolic */
}

- string
{
	return string;
}

- (BOOL) isEqual:anIdentifier
{
        /* comparing nodes */
        if (isa != [anIdentifier class]) return NO;

	if (self == anIdentifier) {
		return YES;
	} else {
		return [[self string] isEqual:[anIdentifier string]];
	}
}

- (unsigned) hash
{
	return [[self string] hash];
}

- (BOOL) isEqualSTR:(STR)s
{
	return [[self string] isEqualSTR:s];
}

- (STR) str
{
	return [[self string] str];
}


static id currentLocalVariables, currentLocalTypes;
static id currentScope;

- doWarnLocalInstance
{
    if ([options warnLocalInstance] && ([self isIVar] || [self isCVar])) {
	id msg;
	msg=[String sprintf:"Definition of '%s' hides instance variable",[self str]];
	warnError([msg str]);
    }

    return self;
}

+ pushScope:aScope
{
    if (!currentScope) currentScope = [Collection new];
    [currentScope add:aScope];
    return self;
}

+ popScope
{
    return [currentScope removeLast];
}

+ pushLocalVars:localVariables types:localTypes
{
    int i,n;

    if (!currentLocalVariables) currentLocalVariables = [Collection new];
    if (!currentLocalTypes)     currentLocalTypes = [Collection new];

    assert([currentLocalVariables size] == [currentLocalTypes size]);

    for(i=0,n=[localVariables size];i<n;i++) {
	id v = [localVariables at:i];
	[v doWarnLocalInstance];
	[currentLocalVariables add:v];
    }

    [currentLocalTypes addContentsOf:localTypes];
    assert([currentLocalVariables size] == [currentLocalTypes size]);

    return self;
}

+ popLocals:localVariables
{
    int n;

    n = (localVariables)?[localVariables size]:0;
    assert([currentLocalVariables size] == [currentLocalTypes size]);

    while (n--) { 
       [currentLocalVariables removeLast];
       [currentLocalTypes removeLast];
    }

    assert([currentLocalVariables size] == [currentLocalTypes size]);
    return self;
}


- outputOn:stream
{
	[LineTag outputOn:stream until:lineTag];
	[stream printObject:string];
	return self;
}

- (BOOL) isSelf
{
    return [ClassDefinition inMethodImplementation] && [self isEqualSTR:"self"];
}

- (BOOL) isLocal
{
    return (currentLocalVariables!=nil &&
	[currentLocalVariables findMatching:self]);
}

- findScopeBlockVar
{
    /* argh! "id aBlock = [ { aBlock } self ];"
     * lhs must not be considered
     * as BlockVar inside statement declarators
     */

    if (noTypeDeclaratorMark == NO) {
	int n = [currentScope size];
	while (n--) {
	    id aScope = [currentScope at:n];
	    if ([[aScope blockVariables] findMatching:self]) return aScope;
	}
    }

    return nil;
}

- (BOOL) isIVar
{
	return [ClassDefinition doesCurrentSetOfIVarsContain:self];
}

- (BOOL) isCVar
{
	return [ClassDefinition doesCurrentSetOfCVarsContain:self];
}

+ translateSelfOn:aStream
{
    id scope;
    static id aSelf;
    if (!aSelf) aSelf = [Identifier str:"self"];

    /*
     * stes - 10/97 in a Block, we need to access 'self' via the
     * block data.
     */

    if ((scope = [aSelf findScopeBlockVar])) {
	[aSelf translateAsBlockVarOf:scope on:aStream];
    } else {
	[aStream printObject:[aSelf string]];
    }

    return self;
}

+ translateSelfAsIdOn:aStream
{
    [aStream echo:"(id)("];
    [self translateSelfOn:aStream];
    [aStream echo:")"];
    return self;
}

- translateAsBlockVarOf:aScope on:stream
{
	[stream echo:"%s->",[[aScope scopeDataName] str]];
	[stream printObject:string];
	return self;
}

- translateOn:stream asIvarOf:aClassname
{
	[Identifier translateSelfOn:stream];
	[stream echo:"->"];
	[stream printObject:string];
	return self;
}

- translateOn:stream asCvarOf:aClass
{
    STR s = [aClass structSharedName];

    /* maybe we could rather use a test to see whether 
     * self->isa is a factory or meta, at runtime
     * (would work for any "self" pointer)
     */

    [stream echo:"((struct %s *)",s];

    if ([currentMethodImplementation isFactory]) {
	[Identifier translateSelfOn:stream];
    } else {
        [stream echo:"("];
	[Identifier translateSelfOn:stream];
        [stream echo:"->isa)"];
    }

    [stream echo:")->",s];

    [stream printObject:string];
    return self;
}

- translateFunctionHeadOn:stream
{
    /* foo(x) translates to foo(x) if foo is ivar */
    /* (*foo)(x) translates to (*self->foo) if foo is ivar */

    [LineTag outputOn:stream until:lineTag];
    [stream printObject:string];
    return self;
} 

- translateFieldOn:stream
{
    /* foo->x translates to foo->x, foo.x to foo.x, if "x" is ivar */

    [LineTag outputOn:stream until:lineTag];
    [stream printObject:string];
    return self;
} 
- translateOn:stream
{
        id scope;

	[LineTag outputOn:stream until:lineTag];

        /* note that local vars take precedence over ivars.
	 * also, local vars of a block take precedence over
	 * blockdata variables (this works, since CompoundStatement
	 * localvars of a Block do NOT contain the blockdata)
	 * (unlike normal compounds inside compounds)
	 */

	if ([self isSelf]) {
	    [Identifier translateSelfAsIdOn:stream]; /* special cast rule */
	} else if ([self isLocal]) {
	    [stream printObject:string]; /* use local, not ivar */
	} else if ([self isIVar]) {
	        id aClass = [ClassDefinition currentName];
		[self translateOn:stream asIvarOf:aClass];
	} else if ([self isCVar]) {
	        id aClass = [ClassDefinition current];
		[self translateOn:stream asCvarOf:aClass];
	} else if ((scope = [self findScopeBlockVar])) {
	   [self translateAsBlockVarOf:scope on:stream];
	} else {
	    if ([self isClassName] && [options sharedData] == NO) {
		[stream echo:"(*%s())",[self classGlobalFun]];
	    } else {
		[stream printObject:string];
	    }
	}

	return self;
}

- outputWithoutTagsOn:stream
{
	[stream echo:" "];
	[stream printObject:string];
	return self;
}

- print
{
	[string printOn:stdout];return self;
}


- markNoTypeDeclarator:(BOOL)flag
{
	noTypeDeclaratorMark = flag;
	return self;
}

static id setOfBuiltinFunctions;

- defineBuiltinFunction
{
	if (!setOfBuiltinFunctions) setOfBuiltinFunctions = [Set new];
	[setOfBuiltinFunctions add:[self dontFree]];
	return self;
}

static id templateTypeNames;

- defineTypeName
{
	if (!setOfTypeNames) setOfTypeNames = [Set new];

	if (noTypeDeclaratorMark) {
	    [setOfTypeNames add:[self dontFree]];
	    if ([options debugInfo]) {
		fprintf(stderr,"defined typename '%s'\n",[self str]);
	    }
	}

	return self;
}

- defineTemplateTypeName
{
	/* hack for C++ templates, where type has limited scope */
	if (!templateTypeNames) templateTypeNames = [Collection new];
	if (noTypeDeclaratorMark) [templateTypeNames add:[self dontFree]];
	return [self defineTypeName];
}

+ (BOOL) inTemplate
{
	return templateTypeNames != nil; /* see scanplus */
}

+ endTemplate
{
	if (templateTypeNames) {
		[setOfTypeNames removeContentsOf:templateTypeNames];
		templateTypeNames = nil;
		return self;
	} else {
		return self;
	}
}

- defineClassName
{
	if (!setOfClassNames) setOfClassNames = [Set new];;
	[setOfClassNames add:[self dontFree]];
	return self;
}

- (BOOL) isBuiltinFunction
{
	if (!setOfBuiltinFunctions) setOfBuiltinFunctions = [Set new];
	return [setOfBuiltinFunctions contains:self];
}

- (BOOL) isTypeName
{
	if (!setOfTypeNames) setOfTypeNames = [Set new];
	return [setOfTypeNames contains:self];
}

- (BOOL) isClassName
{
	if (!setOfClassNames) setOfClassNames = [Set new];
	return [setOfClassNames contains:self];
}

- (STR) classGlobalFun
{
    id z = [String sprintf:"OBJCCLASS_%s",[self str]];return [z str];
}

- (STR) classStructFun
{
    id z = [String sprintf:"%sStruct",[self str]];return [z str];
}

- (STR) metaStructFun
{
    id z = [String sprintf:"%sMeta",[self str]];return [z str];
}

- findIdentifier
{	
	return self;
}


- addIvarsTo:aSet addItype:bNode to:bSet
{
    /* because this can be sent to an initializer ( id o = foo(); )
     * we only want the identifier which was marked ('o')
     */

     if (noTypeDeclaratorMark) {
	[aSet add:[self dontFree]];
	[bSet add:bNode];
     }
     
     return self;
}

- addReferencedScopeToBlock:aBlock
{
     int n = [currentScope size];

     while (n--) {
	 id aScope = [currentScope at:n];
	 if ([aScope referenceBlockVar:self]) {
	     [aBlock addScope:aScope];
	     break;
	 }
     }

     /* within a method implementation, can reference instance
      * variables.  those are used "by reference" (via self pointer)
      * we don't copy the ivar into the Block, but rather the self pointer.
      */
     
     if (currentMethodImplementation) { 
	 /* stes 1/98 we can do this for every identifier
	  * but have to be a bit careful about functions, methods etc.
	  * with names that are ivars.  this is done by overriding
	  * -addReferencedScopeToBlock: in other places -- not here
	  */
         if ([self isIVar] || [self isCVar]) {
	     [[Identifier str:"self"] addReferencedScopeToBlock:aBlock];
	 }
     }

     return self;
}

- (BOOL) containsStatic
{
	return NO;
}

- processForMethod:aMethod
{
	/* case of unary selector, no arguments */

	[aMethod addNameComponent:[self string]];
	return self;
}

@end

