
/*
 * Portable Object Compiler (c) 1997.  All Rights Reserved.
 */

#include "oclib.h"

@implementation Node
/*****************************************************************************
 *
 * Errors
 *
 ****************************************************************************/

int exitStatus = 0;
id setOfTypeNames;
id setOfClassNames;

/* stes - mar 26, 1997
 * change format from
 * objc: syntax error "translate" at test.m:358
 * to
 * test.m:358: [objc1] syntax error at "translate"
 * which is the standard format for emacs and vi
 */

void warnError(STR msg)
{
	STR name    = [LineTag currentFileName]; 
	int lineNo  = [LineTag currentLineNo];
	STR program = ([options cplus])?"objcpls1":"objc1";
   	fprintf(stderr,"%s:%d: warning (%s): %s\n", name,lineNo ,program, msg);
}

void fatalError(STR msg)
{
	STR name    = [LineTag currentFileName]; 
	int lineNo  = [LineTag currentLineNo];
	STR program = ([options cplus])?"objcpls1":"objc1";
   	fprintf(stderr,"%s:%d: error (%s): %s\n", name,lineNo ,program, msg);
	exitStatus = 1;
}

/* utility function to emit code for lhs = rhs assignment
 *
 * we might want to do special stuff here in the future, like
 * emitting a function idAssign(&lhs,rhs) which does reference counting ...
 *
 */

id doAssign(id lhs,id t,id rhs)
{
    id n = [Node new];
    id aClass = [ClassDefinition currentName];

    if ((lhs!=nil) && [lhs isEqualSTR:"self"] &&
     (currentMethodImplementation != nil)) {
	id s;

	if ([options selfAssign]==NO) {
	    fatalError("assignment not allowed with -noSelfAssign");
	}

	/* deliberately do NOT emit the lhs Identifier "self"
	 * which would be translated to (id)self.  it's not ANSI C
	 * to have something like (id) self = ..., and some compilers
	 * (icc or wcc, for instance) complain or warn about this lhs cast
	 */

	s = [String sprintf:"self = (struct %s_PRIVATE *)",[aClass str]];
	[n add:[Identifier string:s]];[n add:rhs];
    } else {
	[n add:lhs];[n add:t];[n add:rhs];
    }
    
    return n;
}

/*****************************************************************************
 *
 * Creation
 *
 ****************************************************************************/


- setUpNode
{
	subnodes = nil; /* created only when needed */
	return self;
}

+ new
{
	return [[super new] setUpNode];
}

- doDeepCopy
{
    subnodes = [subnodes deepCopy];
    return self;
}

- deepCopy
{
    /* not all super's have a deepCopy */
    return [[super copy] doDeepCopy];
}

- free
{
	if (subnodes) subnodes = [[subnodes freeContents] free];
	return [super free];
}


- subnodes
{
	return subnodes;
}

- (BOOL) isEqual:aNode
{
    if (isa != [aNode class]) return NO;

    if (subnodes) {
	int n = [subnodes size];
	id aSubnodes = [aNode subnodes];

	if (aSubnodes == nil || n != [aSubnodes size]) return NO;
	while(n--) if(![[subnodes at:n] isEqual:[aSubnodes at:n]]) return NO;
    }

    return YES;
}

- (BOOL) isEqualSTR:(STR)s
{
	if (subnodes!=nil && [subnodes size] == 1) {
		return [[subnodes at:0] isEqualSTR:s];
	} else {
		return NO;
	}
}

- (int) asInt
{
    if (subnodes!=nil&& [subnodes size]==1) {
	return [[subnodes at:0] asInt];
    } else {
	return 0;
    }
}

/*****************************************************************************
 *
 * Do Semantical Analysis.
 * |process| can be overwritten to do things like typeChecking, defining
 * types etc.
 *
 ****************************************************************************/

- process
{
	return self;
}


/*****************************************************************************
 *
 * Dealing with "typedef".
 * Our scan.l discriminates IDENTIFIER_TOKEN from TYPENAME_TOKEN, by table
 * lookup in a global "setOfTypeNames".
 *
 ****************************************************************************/


- (BOOL) isIdType
{
	if (subnodes && [subnodes size] == 1) {
		return [[subnodes at:0] isIdType];
	}

	return NO;
}

- (BOOL) containsTypedef
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			if ([[subnodes at:i] containsTypedef]) return YES;
		}
	}

	return NO;
}

- defineTypeName
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) [[subnodes at:i] defineTypeName];
	}
	return self;
}

- markNoTypeDeclarator:(BOOL)flag
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) [[subnodes at:i] markNoTypeDeclarator:flag];
	}
	return self;
}

/*****************************************************************************
 *
 * Dealing with temporary variables & inline Caches
 * The messenger needs temporary variables for receivers.
 * Methods to find out whether there's message expressions.
 * These are being sent by the compound statement (with itself as argument).
 *
 ****************************************************************************/


- addTempVarsTo:aStmt
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] addTempVarsTo:aStmt];
		}
	}

	return self;
}

- addBlocksTo:aStmt
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] addBlocksTo:aStmt];
		}
	}

	return self;
}

- addMyBlocksTo:aStmt
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] addMyBlocksTo:aStmt];
		}
	}

	return self;
}

- addInlineCachesTo:aStmt
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] addInlineCachesTo:aStmt];
		}
	}

	return self;
}

- processBlocks
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] processBlocks];
		}
	}

	return self;
}

- outputBlockDefsOn:aStream
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] outputBlockDefsOn:aStream];
		}
	}

	return self;
}

- addReferencedScopeToBlock:aBlock
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] addReferencedScopeToBlock:aBlock];
		}
	}

	return self;
}

/*****************************************************************************
 *
 * Dealing with "main()".
 * We want to insert an extra function call (to initialize the runtime).
 *
 ****************************************************************************/


- findIdentifier
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			id ident = [[subnodes at:i] findIdentifier];
			if (ident) return ident;
		}
	}

	return nil;
}

- (BOOL) containsStatic
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			if ([[subnodes at:i] containsStatic]) return YES;
		}
	}

	return NO;
}


/*****************************************************************************
 *
 * Addding subnodes
 *
 ****************************************************************************/


- add:aNode
{
	if (!subnodes) subnodes = [Collection new];
	/* allow nil nodes for "empty" rules */
	if (aNode) [subnodes add:aNode];
	return self;
}


/*****************************************************************************
 *
 * Assembling instance variables for @interface
 *
 ****************************************************************************/

- addIvarsTo:aSet addItype:bNode to:bSet
{
	/* make a copy so that int *x, y; will store type
	 * int* for x, and type int for y
	 */

        id aCopy = [bNode deepCopy];

	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
		    [[subnodes at:i] addIvarsTo:aSet addItype:aCopy to:bSet];
		}
	}

	return self;
}


/*****************************************************************************
 *
 * Assembling Names, Arguments etc. for Method
 *
 ****************************************************************************/

- processForMethod:aMethod
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] processForMethod:aMethod];
		}
	}

	return self;
}


/*****************************************************************************
 *
 * Output code
 *
 ****************************************************************************/


- outputOn:stream
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] outputOn:stream];
		}
	}

	return self;
}

- outputWithoutTagsOn:stream
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] outputWithoutTagsOn:stream];
		}
	}

	return self;
}

- translateFieldOn:aStream
{
    return [self translateOn:aStream];
}

- translateFunctionHeadOn:aStream
{
    return [self translateOn:aStream];
}

- translateOn:stream
{
	if (subnodes) {
		int i,n = [subnodes size];
		for(i=0;i<n;i++) {
			[[subnodes at:i] translateOn:stream];
		}
	} else {
		[self outputOn:stream];
	}

	return self;
}

- printOn:(IOD)f
{
	id aStream = [OutputStream newFile:f];
	[self outputOn:aStream];return self;
}

@end

