
/*
 * Portable Object Compiler (c) 1997.  All Rights Reserved.
 */

#include "oclib.h"
#include "stdlib.h" /* getenv */

id options;

@implementation Options

/*****************************************************************************
 *
 * Creation
 *
 ****************************************************************************/


/* stes 12/97 - unfortunate choice of me was to use the short form
 * # %d %s as default (as in gcc) ANSI C uses #line, and so most
 * compilers had to be invoked with -longlinetag, which is silly
 */
static char *dfltLineTagFormat = "#line %d \"%s\"";

+ new
{
	if (options) return options;

	options = [super new];
	[options setQuiet:NO];
	[options setOutputCode:YES];
	[options setTranslateCode:YES];
	[options setVersion:NO];
	[options setInFile:nil /* stdin */];
	[options setOutFile:nil /* stdout */];
	/* nil is understood by the LineTag class as "use inFile" */
	[options setSrcFileName:nil];
	[options setLineTagFormat:[String str:dfltLineTagFormat]];
	[options setInitCall:nil]; /* no default - always set at driver level */
	[options setMainFunction:[String str:"main"]];
	[options setDoSelTransTbl:YES];
	[options setInlineCache:NO];
	[options setLineTags:YES];
	 /* default is to do auto-initialization and to use regular "ld" */
	[options setPostLink:NO];
	/* default is to assume shared data is available */
	[options setSharedData:YES];
	[options setRefBind:NO]; /* whether to force a reference to BIND */
	[options setCheckBindCalled:NO]; /* used in selector output */
	[options setWarnClassUsedAsType:YES]; /* warn by default */
	[options setWarnUndefinedMethod:YES]; /* warn by default */
	[options setWarnInterfaceNotFound:YES];
	[options setWarnLocalInstance:YES]; /* local hides instance warn */
	[options setWarnLex:NO]; /* 0xff problem */
	[options setBuffered:YES]; /* use block buffering */
	[options setComments:NO]; /* so that output file can have .i extension */
	[options setMSDOS:NO]; /* by default, no MS-DOS keywords */
	[options setGNU:NO]; /* by default, no GNU keywords */
	[options setCplus:NO]; /* by default, not cplusplus mode */
	[options setDebugInfo:NO]; /* by default, dont' output debug info */
	[options setPPI:NO]; /* by default, no PPI syntax */
	[options setIBMVAC:NO]; /* by default, no Visual Age keywords */
	
	/* default separator for this platform (at compile time) */
	[options setPathSeparator:OBJCRT_DEFAULT_PATHSEPC];
	
	/* default is to allow more than one implementation per class */
	[options setOnePerFile:NO];
	
	/* DEC alpha defines a asm() function... */
	[options setAsmKeyword:YES];
	
	/* SGI IRIX __long_long keyword support */
	[options setLLKeyword:YES];
	
	/* initialization options */
	[options setCthreads:NO];
	[options setPthreads:NO];
	[options setBoehm:NO];
	
	/* allow Objective-C blocks extensions by default */
	[options setBlocks:YES];
	
	/* link the AsciiFiler class into program */
	[options setFilerCode:YES];
	
	/* do not by default emit __declspec(export) for OBJCBIND */
	[options setBind:[String new]];

	/* no max by default */
	[options setLineMax:0];

	/* objcrt control */
	[options setCache:YES];
	[options setNilRcvr:YES];

	/* by default, allow self= */
	[options setSelfAssign:YES];

	/* purists can set this to NO and use +addMethodsTo:  */
	[options setCats:YES];

	return options;
}


- setLineMax:(int)flag
{
     lineMax = flag;
     return self;
}

- (int) lineMax
{
     return lineMax ;
}

- setCats:(BOOL)flag
{
    cats = flag;
    return self;
}

- (BOOL) cats
{
    return cats;
}

- setSelfAssign:(BOOL)flag
{
    selfAssign = flag;
    return self;
}

- (BOOL) selfAssign
{
    return selfAssign;
}

- setRefBind:(BOOL)flag
{
    /* whether to force a reference to the OBJCBIND function
     * needed on Metrowerks compiler
     */
    refBind = flag;
    return self;
}

- (BOOL) refBind
{
    return refBind;
}

- setNilRcvr:(BOOL)flag
{
    nilRcvr = flag;
    return self;
}

- (BOOL) nilRcvr
{
    return nilRcvr;
}

- setCache:(BOOL)flag
{
    cache = flag;
    return self;
}

- (BOOL) cache
{
    return cache;
}

- setBind:arg
{
    bind = arg;
    return self;
}

- bind
{
    return bind;
}

- setFilerCode:(BOOL)flag
{
    filerCode = flag;
    return self;
}

- (BOOL) filerCode
{
    return filerCode;
}

- setBlocks:(BOOL)flag
{
    blocks = flag;
    return self;
}

- (BOOL) blocks
{
    return blocks;
}

- setPthreads:(BOOL)flag
{
    pthreads = flag;
    return self;
}

- (BOOL) pthreads
{
    return pthreads;
}

- setCthreads:(BOOL)flag
{
    cthreads = flag;
    return self;
}

- (BOOL) cthreads
{
    return cthreads;
}

- setBoehm:(BOOL)flag
{
    boehm = flag;
    return self;
}

- (BOOL) boehm
{
    return boehm;
}

- setLLKeyword:(BOOL)flag
{
    llKeyword = flag;
    return self;
}

- (BOOL) llKeyword
{
    return llKeyword;
}

- setAsmKeyword:(BOOL)flag
{
    asmKeyword = flag;
    return self;
}

- (BOOL) asmKeyword
{
    return asmKeyword;
}

- setOnePerFile:(BOOL)flag
{
    onePerFile = flag;
    return self;
}

- (BOOL) onePerFile
{
    return onePerFile;
}

- setQuiet:(BOOL)flag
{
    quiet = flag;
    return self;
}

- (BOOL) quiet
{
    return quiet;
}

- setPathSeparator:(STR)aSep
{
	pathSeparator = (STR)getenv("OBJCPATHSEP");
	pathSeparator = (pathSeparator)?pathSeparator:aSep;
    return self;
}

- (STR) pathSeparator
{
	return pathSeparator;
}

- setPPI:(BOOL)flag
{
    ppi = flag;
    return self;
}

- (BOOL) ppi
{
    return ppi;
}

- setDebugInfo:(BOOL)flag
{
	debugInfo = flag;
    return self;
}

- (BOOL) debugInfo
{
    return debugInfo;
}

- setCplus:(BOOL)flag
{
    cplus = flag;
    return self;
}

- (BOOL) cplus
{
    return cplus;
}

- setMSDOS:(BOOL)flag
{
    msdos = flag;
    return self;
}

- (BOOL) msdos
{
    return msdos;
}

- setWATCOM:(BOOL)flag
{
    watcom = flag;
    return self;
}

- (BOOL) watcom
{
    return watcom;
}

- setIBMVAC:(BOOL)flag
{
    ibmvac = flag;
    return self;
}

- (BOOL) ibmvac
{
    return ibmvac;
}

- setGNU:(BOOL)flag
{
    gnu = flag;
    return self;
}

- (BOOL) gnu
{
    return gnu;
}

- setComments:(BOOL)flag
{
    comments = flag;
    return self;
}

- (BOOL) comments
{
    return comments;
}

- setOutputCode:(BOOL)flag
{
    outputCode = flag;
    return self;
}

- (BOOL) outputCode
{
    return outputCode;
}

- setDoSelTransTbl:(BOOL)flag
{
    doSelTransTbl = flag;
    return self;
}

- (BOOL) doSelTransTbl
{
    return doSelTransTbl;
}

- setTranslateCode:(BOOL)flag
{
    translateCode = flag;
    return self;
}

- (BOOL) translateCode
{
    return outputCode;
}

- setBuffered:(BOOL)flag
{
    isBuffered = flag;
    return self;
}

- (BOOL) isBuffered
{
    return isBuffered;
}

- setLineTags:(BOOL)flag
{
    lineTags = flag;
    return self;
}

- (BOOL) lineTags
{
    return lineTags;
}

- setSharedData:(BOOL)flag
{
    sharedData = flag;
    return self;
}

- (BOOL) sharedData
{
    return sharedData;
}

- setVersion:(BOOL)flag
{
    version = flag;
    return self;
}

- (BOOL) version
{
    return version;
}

- setInlineCache:(BOOL)flag
{
    inlineCache = flag;
    return self;
}

- (BOOL) inlineCache
{
    return inlineCache;
}

- setWarnLex:(BOOL)flag
{
    warnLex = flag;
    return self;
}

- (BOOL) warnLex
{
    return warnLex;
}

- setWarnClassUsedAsType:(BOOL)flag
{
    warnClassUsedAsType = flag;
    return self;
}

- (BOOL) warnClassUsedAsType
{
    return warnClassUsedAsType;
}

- setWarnInterfaceNotFound:(BOOL)flag
{
    warnInterfaceNotFound = flag;
    return self;
}

- (BOOL) warnInterfaceNotFound
{
    return warnInterfaceNotFound;
}

- setWarnLocalInstance:(BOOL)flag
{
    warnLocalInstance = flag;
    return self;
}

- (BOOL) warnLocalInstance
{
    return warnLocalInstance;
}

- setWarnUndefinedMethod:(BOOL)flag
{
    warnUndefinedMethod = flag;
    return self;
}

- (BOOL) warnUndefinedMethod
{
    return warnUndefinedMethod;
}

- setCheckBindCalled:(BOOL)flag
{
    checkBindCalled = flag;
    return self;
}

- (BOOL) checkBindCalled
{
    return checkBindCalled;
}

- setPostLink:(BOOL)flag
{
    postLink = flag;
    return self;
}

- (BOOL) postLink
{
    return postLink;
}

- setInFile:aString
{
    inFile = aString;
    return self;
}

- inFile
{
    return inFile;
}

- setSrcFileName:aString
{
    srcFileName = aString;
    return self;
}

- srcFileName
{
    return srcFileName;
}

- setOutFile:aString
{
    outFile = aString;
    return self;
}

- outFile
{
    return outFile;
}

- setLineTagFormat:aString
{
    lineTagFormat = aString;
    return self;
}

- lineTagFormat
{
    return lineTagFormat;
}

- setInitCall:aString
{
    initCall = aString;
    return self;
}

- initCall
{
    return initCall;
}

- setMainFunction:aString
{
    mainFunction = aString;
    return self;
}

- mainFunction
{
    return mainFunction;
}


@end

